=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-12-13 17:37:07 +0000
+++ bzrlib/bzrdir.py	2006-12-21 10:28:19 +0000
@@ -505,6 +505,10 @@
         :param transport: Transport containing the bzrdir.
         :param _unsupported: private.
         """
+        # FIXME: Put that outside the static method once I
+        # understand how to do it.
+        MAX_REDIRECTIONS = 8
+
         initial_base = transport.base
         redirected = True # to enter the loop
         redirections = 0
@@ -512,15 +516,11 @@
         # don't try to detect them, just getting out if too much
         # redirections occurs. The solution is outside: where the
         # loop is defined.
-        while redirected and redirections < 8:
+        while redirected and redirections < MAX_REDIRECTIONS:
             try:
-                transport.follow_redirections = False
                 format = BzrDirFormat.find_format(transport)
                 redirected = False
             except errors.RedirectRequested, e:
-                # In case someone above in the call stack want to
-                # continue using this transport
-                transport.follow_redirections = True
 
                 relpath = transport.relpath(e.source)
                 if not e.target.endswith(relpath):
@@ -1100,8 +1100,9 @@
     def probe_transport(klass, transport):
         """Return the .bzrdir style transport present at URL."""
         try:
-            format_string = transport.get(
-                ".bzr/branch-format", hints={'follow_redirections':False}).read()
+            format_file = transport.get(".bzr/branch-format",
+                                        hints={'follow_redirections':False})
+            format_string = format_file.read()
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base)
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/__init__.py	2006-12-21 10:28:19 +0000
@@ -81,6 +81,107 @@
         warnings.warn("register_transport(override) is deprecated")
     _protocol_handlers.setdefault(prefix, []).insert(0, klass)
 
+def _get_prototype_infos(func):
+    """Acquire prototype informations about a function by introspection.
+
+    :param func: the function we want to examine
+
+    :return: (name, params, default_of) A tuple with the name,
+             the parameters and the parameters default
+             values. The later is a dict for ease of use.
+    """
+    # Only the first local variables are the parameters of the
+    # function
+    code = func.func_code
+    local_variables = code.co_varnames
+    nb_params = code.co_argcount
+    func_params = local_variables[:nb_params]
+    defaults = func.func_defaults
+    default_of = {}
+    if defaults:
+        # Only the last params can have a default value
+        for name, value in map(None, func_params[nb_params-len(defaults):],
+                               defaults):
+            default_of[name] = value
+
+    return func.__name__, func_params, default_of
+
+def _build_arglist(args, default_of={}, build_default=None):
+    arglist = ''
+    for arg in args:
+        if arglist: arglist += ', ' # comma separated if more than one
+        arglist += arg
+        if default_of.has_key(arg):
+            arglist += '='
+            if build_default:
+                arglist += build_default(default_of, arg)
+            else:
+                arglist += '%r' % default_of[arg]
+    return arglist
+
+def _inject_get_with_hints(klass, get_params, default_of):
+    """Provide a new get method with an hints parameter.
+
+    :param klass: the klass ignorant of hints
+    :param get_params: the actual parameters of the get method
+    :param default_of: the optional default values of the above parameters
+    """
+    # First we prepare the new prototype (building the old one
+    # partially in parallel)
+    old_params = _build_arglist(get_params[1:]) # Avoid self
+
+    # We want to evaluate this prototype later, in a context
+    # where we can access the default values, whatever they are
+    # (i.e. we could have embedded constants directly but a
+    # default value can be an object, so we really need to access
+    # its value, not its stringified representation).  To achieve
+    # that, we need to differ the evalation of the default value
+    # and we use an hopefully hard to conflict with dictionary
+    # name (four trailing '_', now try to conflict with *that* !)
+    new_params = _build_arglist(get_params, default_of,
+                                lambda default_of, arg: \
+                                "default_of____['%s']" % arg)
+
+    # Add the new hints parameter with its default value
+    new_params += ', hints={}'
+    # Prepare the injection
+    kmodule = klass.__module__
+    kname = klass.__name__
+    code = """
+from %(kmodule)s import %(kname)s
+
+def get(%(new_params)s):
+    return self.get_without_hints(%(old_params)s)
+
+%(kname)s.get_without_hints = %(kname)s.get
+%(kname)s.get = get
+"""
+    default_of____ = default_of # Don't forget that one
+    code = code % locals() # So we can access our just cooked strings
+
+    # Compilation and evaluation are necessary to achieve the
+    # desired effect: declare a new sub and evaluate its
+    # default values in the right context.
+    mutter('Will inject: %s' % code)
+    eval(compile(code, '<compatibility kitchen>', 'exec'))
+
+def _check_get_with_hints(klass):
+    kget = klass.get
+    kgetwh = getattr(klass,'get_without_hints',None)
+
+    if kgetwh is None: # Don't try to inject twice
+        name, params, default_of = _get_prototype_infos(kget)
+        if 'self' == params[0] and not 'hints' in params:
+            # Looks like we have a candidate here
+            warning("%r do not specify 'hints'" \
+                    "in the %s(%s) method prototype" \
+                    % (klass, name, _build_arglist(params, default_of)))
+
+            _inject_get_with_hints(klass, params, default_of)
+
+            name, params, default_of = _get_prototype_infos(klass.get)
+            mutter('in %r, proto is now: %s(%s)',
+                   klass, name, _build_arglist(params, default_of))
 
 def register_lazy_transport(scheme, module, classname):
     """Register lazy-loaded transport class.
@@ -100,6 +201,7 @@
     def _loader(base):
         mod = __import__(module, globals(), locals(), [classname])
         klass = getattr(mod, classname)
+        _check_get_with_hints(klass)
         return klass(base)
     _loader.module = module
     register_transport(scheme, _loader)
@@ -131,7 +233,7 @@
         for factory in factory_list:
             if factory.__module__ == "bzrlib.transport":
                 # this is a lazy load transport, because no real ones
-                # are directlry in bzrlib.transport
+                # are directly in bzrlib.transport
                 modules.add(factory.module)
             else:
                 modules.add(factory.__module__)

=== modified file 'bzrlib/transport/chroot.py'
--- bzrlib/transport/chroot.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/chroot.py	2006-12-21 10:28:19 +0000
@@ -71,7 +71,8 @@
 
     def get(self, relpath, hints={}):
         self._ensure_relpath_is_child(relpath)
-        return TransportDecorator.get(self, relpath)
+        # FIXME: Is it wise to forward the follow_redirection hint ?
+        return TransportDecorator.get(self, relpath, hints)
 
     def get_bytes(self, relpath):
         self._ensure_relpath_is_child(relpath)

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/ftp.py	2006-12-21 10:28:19 +0000
@@ -252,7 +252,8 @@
             mutter("FTP has not: %s: %s", abspath, e)
             return False
 
-    def get(self, relpath, decode=False, retries=0, hints={}):
+    # FIXME: the 'hints' parameter is commented out
+    def get(self, relpath, decode=False, retries=0):#, hints={}):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-12-21 10:28:19 +0000
@@ -152,7 +152,7 @@
 
         return abspath, data, header
 
-    def _get_full(self, relpath, hints):
+    def _get_full(self, relpath, hints={}):
         """Make a request for the entire file"""
         curl = self._curl
         abspath, data, header = self._setup_get_request(curl, relpath)

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/http/_urllib.py	2006-12-21 10:28:19 +0000
@@ -135,7 +135,7 @@
                 headers = {'Range': bytes}
 
         request = Request('GET', abspath, None, headers)
-        request.follow_redirections = getattr(hints, 'follow_redirections', True)
+        request.follow_redirections = hints.get('follow_redirections', True)
         response = self._perform(request)
 
         code = response.code

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/local.py	2006-12-21 10:28:19 +0000
@@ -126,7 +126,8 @@
     def has(self, relpath):
         return os.access(self._abspath(relpath), os.F_OK)
 
-    def get(self, relpath, hints={}):
+    # FIXME: the 'hints' parameter is commented out
+    def get(self, relpath):#, hints={}):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-12-13 17:37:07 +0000
+++ bzrlib/transport/smart.py	2006-12-21 10:28:19 +0000
@@ -1038,6 +1038,7 @@
         else:
             self._translate_error(resp)
 
+    # FIXME: 'hints' is lost ?
     def get(self, relpath, hints={}):
         """Return file-like object reading the contents of a remote file.
         

